--
-- MPOwner
--
-- @author  Destroileur, ANDR55
-- Version 1.0
-- @date 20.10.2018

MPOwner_ui = {};
MPOwner_ui.modDir = g_currentModDirectory;

-- character table string
local b='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';

-- encoding
function enc(data)
    return ((data:gsub('.', function(x)
        local r,b='',x:byte()
        for i=8,1,-1 do r=r..(b%2^i-b%2^(i-1)>0 and '1' or '0') end
        return r;
    end)..'0000'):gsub('%d%d%d?%d?%d?%d?', function(x)
        if (#x < 6) then return '' end
        local c=0
        for i=1,6 do c=c+(x:sub(i,i)=='1' and 2^(6-i) or 0) end
        return b:sub(c+1,c+1)
    end)..({ '', '==', '=' })[#data%3+1])
end;


-- decoding
function dec(data)
    data = string.gsub(data, '[^'..b..'=]', '')
    return (data:gsub('.', function(x)
        if (x == '=') then return '' end
        local r,f='',(b:find(x)-1)
        for i=6,1,-1 do r=r..(f%2^i-f%2^(i-1)>0 and '1' or '0') end
        return r;
    end):gsub('%d%d%d?%d?%d?%d?%d?%d?', function(x)
        if (#x ~= 8) then return '' end
        local c=0
        for i=1,8 do c=c+(x:sub(i,i)=='1' and 2^(8-i) or 0) end
        return string.char(c)
    end))
end

function money_format(amount)
    local formatted = amount;
    while true do
        formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1.%2')
        if (k==0) then
            break
        end
    end
    return formatted;
end;

function cleanup_string(str)
    str = str:gsub( "%W", "_" );
    str = str:gsub("%d", "_");
	str = str:gsub(",", "");
	str = str:gsub("|", "");
	str = str:gsub("#", "");
    return str;
end;

function MPOwner_ui:fLog(str)

    print(str);

end;


function MPOwner_ui:loadMap(name)

	self.useTimer = 0;
	self.incID = 1;
	self.canEdit = false;
	self.usePrice = 0;
	self.username = "";
	self.masterUsername = "";
	self.ownerID = "";
	self.parent = {};
	self.listPlayers = "";
	self.rules = "";
	self.currentInVehicle = false;
	self.canPopup = false;
	self.ignoreTimerReset = false;
	self.userChecked = false;
	
	self.HUDActive = false;
	self.HUD = {}
    self.HUD.background = createImageOverlay(MPOwner_ui.modDir.."img/"..g_i18n:getText("IMG_BACKGROUND")..".dds");
	
	self.dbFileFound = false;
	self.filedir = getUserProfileAppPath();
    self.dbfilename = self.filedir.."mpowner_db.xml";
	getFiles(self.filedir, "getFilesCallback", self);
	
	self.mpmoney_dbFileFound = false;
	self.mpmoney_filedir = getUserProfileAppPath();
    self.mpmoney_dbfilename = self.mpmoney_filedir.."mpmoney_db.xml";
	
	self.params = {};
	self.params.usePrice = 0;
	self.params.canUse = false;
	self.params.canEdit = false;
	self.params.selectedIndex = 1;
    self.params.selectedNickname = "";
	self.params.currentEditPrice = false;
	
	self.timer = addTimer(1000, "checkTime", self);
	self.timer = addTimer(3000, "delayedCheckFile", self);
	
end;

function MPOwner_ui:delayedCheckFile()
	getFiles(self.mpmoney_filedir, "mpmoney_getFilesCallback", self);
end;

function MPOwner_ui:checkTime()


	if (self.useTimer == 60) then
		self:checkPriceUsage();
	end;
	if (self.currentInVehicle) then
		self.useTimer = self.useTimer+1;
	end;
	self.currentInVehicle = false;
	
	return true;
	
end;


function MPOwner_ui:checkPriceUsage()
	if (self.currentInVehicle and self.usePrice > 0 and self.useTimer > 0) then
		self:SendToServer(self.username, "useEnd",self.ownerID..","..self.useTimer);
	end;
	self.useTimer = 0;
end;

function MPOwner_ui:refresh()
	self.canEdit = false;
	self:SendToServer(self.username, "needRefreshRules", self.ownerID);
end;


function MPOwner_ui:checkDataFile()
	if not self.dbFileFound then
		self:createDataFile();
		self:fLog("## MPOwner ## Create new database file");
    end;
end;


function MPOwner_ui:getFilesCallback(filename)
    if filename == "mpowner_db.xml" then
        self.dbFileFound = true;
		self:fLog("## MPOwner ## Database file found");
    end;
end;

function MPOwner_ui:mpmoney_getFilesCallback(filename)
    if filename == "mpmoney_db.xml" then
        self.mpmoney_dbFileFound = true;
		self:fLog("## MPOwner ## MPMoney database file found");
    end;
end;


function MPOwner_ui:createDataFile()
    local xml = createXMLFile("mpowner_db", self.dbfilename, "mpowner_db");
    saveXMLFile(xml);
    delete(xml);
    self.dbFileFound = true;
end;




function MPOwner_ui:getSelectIndexNickname(index)
    local indexOther = 0;
    for player in string.gmatch(self.listPlayers, "([^#]+)") do
		indexOther = indexOther + 1;
		if index == indexOther then
			return player;
		end;
    end;
    return "";
end;


function MPOwner_ui:getCountNbOther()
    local nbOther = 0;
    for player in string.gmatch(self.listPlayers, "([^#]+)") do
        nbOther = nbOther + 1;
    end;
    return nbOther;
end;



function MPOwner_ui:saveRules()

	local newRules = "";
	local first = true;
	local currentSaved = false;
	for rule in string.gmatch(self.rules, "([^#]+)") do
		local ruleNickname, canUse, usePrice, canEdit = rule:match("([^,]+)|([^,]+)|([^,]+)|([^,]+)");
		if not (ruleNickname == self.params.selectedNickname) then
			if not (ruleNickname == "") then
				if (first == false) then
					newRules = newRules.."#";
				end;
				first = false;
				newRules = newRules..rule;
			end;
		end;
	end;
	
	if (first == false) then
		newRules = newRules.."#";
	end;
	newRules = newRules..self.params.selectedNickname.."|";
	if (self.params.canUse) then
		newRules = newRules.."1|";
	else
		newRules = newRules.."0|";
	end;
	newRules = newRules..self.params.usePrice.."|";
	if (self.params.canEdit) then
		newRules = newRules.."1";
	else
		newRules = newRules.."0";
	end;
	self.rules = newRules;
	self:SendToServer(self.username, "saveRules",self.ownerID..","..self.rules);
	self.canPopup = true;
	
end;

function MPOwner_ui:getSelectNicknameRules(nickname)
	
	self.params.canEdit = false;
	self.params.canUse = false;
	self.params.usePrice = 0;
	
	for rule in string.gmatch(self.rules, "([^#]+)") do
		local ruleNickname, canUse, usePrice, canEdit = rule:match("([^,]+)|([^,]+)|([^,]+)|([^,]+)");
		if (ruleNickname == nickname) then
			if (canEdit == "1") then
				self.params.canEdit = true;
			end;
			if (canUse == "1") then
				self.params.canUse = true;
			end;
			self.params.usePrice = usePrice;
			return true;
		end;
	end;
	

end;


function MPOwner_ui:update(dt)

	if (g_gui.currentGui == nil and self.username == "") then
		-- Get username
		for a=1, table.getn(g_currentMission.users) do
			local user = g_currentMission.users[a];
			if user.userId == g_currentMission.playerUserId then
				self.username = user.nickname;
				self.username = cleanup_string(self.username);
				break;
			end;
		end;
		-- Check if master
        if g_currentMission:getIsServer() then
            self.masterUsername = self.username;
        end;
		if (not g_currentMission:getIsServer()) then
			-- Send user to server
			--self:SendToServer("%All%", "checkMe","");
			self:SendToServer(self.username, "checkMe","");
			self.userChecked = true;
		end;
		-- request refresh all vehicle rules
		for vehicle = 1, table.getn(g_currentMission.steerables) do
			local veh = g_currentMission.steerables[vehicle];
			if not (veh.MPOwner == nil) then
				--print("Current ID:"..veh.MPOwner.ownerID);
				self:SendToServer(self.username, "needRefreshRules", veh.MPOwner.ownerID);
			end;
		end;
	
	end;
	
	if self.HUDActive then
        if InputBinding.hasEvent(InputBinding.CLOSE) then
            self:leaveMenu();
        end;
		if not (g_gui.currentGui == self) then
			self:leaveMenu();
		end;
    end;

end;


function MPOwner_ui:enterMenu()

    if (self.HUDActive == false) then 
		self.HUDActive = true;
        InputBinding.setShowMouseCursor(true);
        g_gui.currentGui = self;
		self.ignoreTimerReset = true;
		self:SendToServer(self.username, "needRefreshPlayers", "");
		self:SendToServer(self.username, "needRefreshRules", self.ownerID);
    end;

end;


function MPOwner_ui:leaveMenu()
    if (self.HUDActive) then
        self.HUDActive = false;
        InputBinding.setShowMouseCursor(false);
        g_gui.currentGui = nil;
    end;
end;


function MPOwner_ui:draw()

	if self.HUDActive then
   
		renderOverlay(self.HUD.background, 0, 0, 1, 1);
		
		-- local fontSize = 0.070;
		-- setTextColor(1, 1, 1, 1);
		-- renderText(0.3198, 0.7703, fontSize, g_i18n:getText("FORM_TITLE"));
		
		-- local fontSize = 0.035;
		-- renderText(0.1499, 0.6657, fontSize, g_i18n:getText("USER"));
		-- renderText(0.1499, 0.5210, fontSize, g_i18n:getText("CAN_USE"));
		-- renderText(0.1499, 0.3890, fontSize, g_i18n:getText("USER_PRICE"));
		-- renderText(0.1499, 0.2549, fontSize, g_i18n:getText("CAN_EDIT"));
		-- renderText(0.1938, 0.4604, fontSize, g_i18n:getText("YES"));
		-- renderText(0.2939, 0.4604, fontSize, g_i18n:getText("NO"));
		-- renderText(0.1938, 0.2023, fontSize, g_i18n:getText("YES"));
		-- renderText(0.2939, 0.2026, fontSize, g_i18n:getText("NO"));
		
		local fontSize = 0.036;
		setTextColor(1, 0.70, 0, 1);
		renderText(0.1601, 0.3255, fontSize, tostring(money_format(math.floor(self.params.usePrice))));
		
		setTextColor(0, 1, 1, 1);
		renderText(0.1601, 0.5982, fontSize, self.params.selectedNickname);
		if (self.params.canUse) then
			renderText(0.1574, 0.4590, fontSize, "X");
		else 
			renderText(0.2580, 0.4590, fontSize, "X");
		end
		if (self.params.canEdit) then
			renderText(0.1574, 0.2027, fontSize, "X");
		else
			renderText(0.2580, 0.2027, fontSize, "X");
		end;
		
		--renderText(0.4000, 0.2000, fontSize, tostring(self.useTimer));
		--renderText(0.4000, 0.2000, fontSize, tostring(self.ownerID));
			
    end;

end;


function MPOwner_ui:mouseEvent(posX, posY, isDown, isUp, button)

	if self.HUDActive then
        if isUp then
		
			self.params.currentEditPrice = false;
		
			if posX > 0.8242 and posX < 0.8701 and posY > 0.7536 and posY < 0.8445 then
				self:leaveMenu();
			end;
		
			-- next user
			if posX > 0.4008 and posX < 0.4296 and posY > 0.5835 and posY < 0.6412 then
				local nbOther = self:getCountNbOther();
				self.params.selectedIndex = math.min(self.params.selectedIndex + 1, nbOther);
				self.params.selectedNickname = self:getSelectIndexNickname(self.params.selectedIndex);
				self:getSelectNicknameRules(self.params.selectedNickname);
			end;

			-- before user
			if posX > 0.3696 and posX < 0.3979 and posY > 0.5835 and posY < 0.6412 then
				self.params.selectedIndex = math.max(self.params.selectedIndex - 1, 1);
				self.params.selectedNickname = self:getSelectIndexNickname(self.params.selectedIndex);
				self:getSelectNicknameRules(self.params.selectedNickname);
			end;
			
			-- Use => Yes
			if posX > 0.1484 and posX < 0.1767 and posY > 0.4447 and posY < 0.5014 then
				self.params.canUse = true;
				self:saveRules();
			end;
			
			-- Use => No
			if posX > 0.2485 and posX < 0.2763 and posY > 0.4486 and posY < 0.5034 then
				if not (self.params.selectedNickname == self.username) then
					self.params.canUse = false;
					self.params.canEdit = false;
					self:saveRules();
				else
					if (self:isMaster(self.username)) then
						self.params.canUse = false;
						self.params.canEdit = false;
						self:saveRules();
					end;
				end;
			end;
			
			-- Edit price
			if posX > 0.1474 and posX < 0.2773 and posY > 0.3157 and posY < 0.3714 then
				self.params.currentEditPrice = true;
			end;
			
			-- Edit => Yes
			if posX > 0.1484 and posX < 0.1767 and posY > 0.1896 and posY < 0.2434 then
				self.params.canUse = true;
				self.params.canEdit = true;
				self:saveRules();
			end;
			
			-- Edit => No
			if posX > 0.2495 and posX < 0.2778 and posY > 0.1896 and posY < 0.2453 then
				if not (self.params.selectedNickname == self.username) then
					self.params.canEdit = false;
					self:saveRules();
				else
					if (self:isMaster(self.username)) then
						self.params.canEdit = false;
						self:saveRules();
					end;
				end;
			end;
				

        end;
    end;
 
end;


function MPOwner_ui:isMaster(username)

	if not (username == "") then
		if g_currentMission:getIsServer() then
			self.masterUsername = self.username;
		end;
		if (username == self.masterUsername) then
			return true;
		end;
		for a=1, table.getn(g_currentMission.users) do
			if g_currentMission.users[a].nickname == username then
				if g_currentMission.users[a].isMasterUser then
					self.masterUsername = username;
					return true;
				end;
			end;
		end;
	end;
    return false;
	
end;



function MPOwner_ui:SendToClient(actionTarget, actionType, actionData)

	if g_currentMission:getIsServer() then
		if (actionTarget == self.username or actionTarget == "%all") then
			self:ClientController(actionTarget, actionType, actionData);
		end;
		if not (type(actionTarget)=="string")then
			actionTarget = tostring(actionTarget);
		end;
		if not (type(actionType)=="string")then
			actionType = tostring(actionType);
		end;
		if not (type(actionData)=="string")then
			actionData = tostring(actionData);
		end;
		ClientControllerEvent.sendEvent(actionTarget, actionType, actionData);
	else
		self:fLog("## MPOwner ## ERROR : Can not send client to client !");
	end;

end;


function MPOwner_ui:SendToServer(actionSource, actionType, actionData)

	if g_currentMission:getIsServer() then
		self:ServerController(actionSource, actionType, actionData);
	else
		if not (type(actionSource)=="string")then
			actionSource = tostring(actionSource);
		end;
		if not (type(actionType)=="string")then
			actionType = tostring(actionType);
		end;
		if not (type(actionData)=="string")then
			actionData = tostring(actionData);
		end;
		ServerControllerEvent.sendEvent(actionSource, actionType, actionData);
	end

end;


function MPOwner_ui:ClientController(actionTarget, actionType, actionData)

	--print("MPOwner_ui:ClientController(actionTarget, actionType, actionData)",actionTarget, actionType, actionData);
	
	if (actionTarget == self.username or actionTarget == "%all") then
		if actionType == "refreshPlayers" then
			self.listPlayers = actionData;
			self.params.selectedNickname = self:getSelectIndexNickname(self.params.selectedIndex);
		elseif actionType == "refreshID" then
			-- Request refresh all vehicle rules
			for vehicle = 1, table.getn(g_currentMission.steerables) do
				local veh = g_currentMission.steerables[vehicle];
				if not (veh.MPOwner == nil) then
					self:SendToServer(self.username, "needRefreshRules", veh.MPOwner.ownerID);
				end;
			end;
		elseif actionType == "refreshRules" then
		
			local ownerID, rules = actionData:match("([^,]+),([^,]+)");
			
			if (string.len(ownerID)>0) then
			
				if (ownerID == self.ownerID) then
					self.rules = rules;
					self.canPopup = true;
					if (self.rules == "none") then
						-- self.rules = "S#"..self.username.."|1|0|1";
						-- self.canEdit = true;
						-- self:saveRules();
						self.rules = "";
						self.params.selectedNickname = self.username;
						self.params.canEdit = true;
						self.params.canUse = true;
						self.params.usePrice = 0;
						self:saveRules();
						g_currentMission:showBlinkingWarning(g_i18n:getText("YOU_ARE_NOW_OWNER"), 3000);
					end;
					self:getSelectNicknameRules(self.params.selectedNickname);
				end;
		
				local canUse = false;
				local usePrice = 0;
				local canEdit = false;
				-- local ownerExist = false;

				for rule in string.gmatch(rules, "([^#]+)") do
					local ruleNickname, db_canUse, db_usePrice, db_canEdit = rule:match("([^,]+)|([^,]+)|([^,]+)|([^,]+)");
					-- if (db_canEdit == "1") then
						-- ownerExist = true;
					-- end;
					if (ruleNickname == self.username) then
						if (db_canUse == "1") then
							canUse = true;
						end;
						if (db_canEdit == "1") then
							canEdit = true;
						end;
						usePrice = db_usePrice;
						break;
					end;
				end;
				
				-- if (ownerExist == false) then
					-- canEdit = true;
					-- canUse = true;
					-- usePrice = 0;
				-- end;
				
				if (ownerID == self.ownerID) then
					self.canEdit = canEdit;
					self.usePrice = tonumber(usePrice);
					if not (self.ignoreTimerReset) then
						self.useTimer = -5;
					end;
					self.ignoreTimerReset = false;
					if (self.usePrice > 0 and self.canPopup and self.currentInVehicle) then
						g_currentMission:showBlinkingWarning(g_i18n:getText("THIS_VEHICLE_COST").." "..self.usePrice.." "..g_i18n:getText("BY_MINUTE"), 3000);
						self.canPopup = false;
					end;
				end;

				for vehicle = 1, table.getn(g_currentMission.steerables) do
					local veh = g_currentMission.steerables[vehicle];
					if not (veh.MPOwner == nill) then
						if (veh.MPOwner.ownerID == ownerID) then
							veh.MPOwner.canUse = canUse;
							veh.MPOwner.canEdit = canEdit;
							veh.MPOwner.usePrice = usePrice;
							break;
						end;
					end;
				end;
			
			end;
			
		else
			self:fLog("## MPOwner ## ERROR : Invalid action '"..actionType.."' from server");
		end;
	end;
	
end;


function MPOwner_ui:ServerController(actionSender, actionType, actionData)

	--print("MPOwner_ui:ServerController(actionSender, actionType, actionData)",actionSender, actionType, actionData);
	
	-- Check if database already exist
	self:checkDataFile();
	
	-- Get data file -- Get data file
    local xml = loadXMLFile("mpowner_db", self.dbfilename);
	
	if actionType == "checkMe" then
		
		if not (actionSender == "") then

			local listPlayers = Utils.getNoNil(getXMLString(xml, "mpowner_db.players"), "");
			listPlayers = dec(listPlayers);

			local usernameExist = false;
			local isFirst = true;
			for player in string.gmatch(listPlayers, "([^#]+)") do
				isFirst = false;
				if (player:lower() == actionSender:lower()) then
					usernameExist = true;
					break;
				end;
			end;

			if not usernameExist then

				if not isFirst then
					listPlayers = listPlayers.."#";
				end;
				listPlayers = listPlayers..actionSender

				listPlayers = enc(listPlayers);

				setXMLString(xml, "mpowner_db.players", listPlayers);
				saveXMLFile(xml);

				self:fLog("## MPOwner ## Add new player '"..actionSender.."' in database");

				self:SendToServer(self.username, "needRefreshPlayers", "");

			end;

		end;
		
	elseif actionType == "needRefreshPlayers" then

			local listPlayers = Utils.getNoNil(getXMLString(xml, "mpowner_db.players"), "");
			listPlayers = dec(listPlayers);
			self:SendToClient("%all", "refreshPlayers", listPlayers);

	elseif actionType == "needRefreshRules" then
		
		local rules = Utils.getNoNil(getXMLString(xml, "mpowner_db.ID"..actionData), enc("none"));
		rules = dec(rules);
		self:SendToClient("%all", "refreshRules", actionData..","..rules);
		
	elseif actionType == "useEnd" then
		
		if (self.mpmoney_dbFileFound) then
		
			local ownerID, useTime = actionData:match("([^,]+),([^,]+)");
			useTime = tonumber(useTime);
			local rules = Utils.getNoNil(getXMLString(xml, "mpowner_db.ID"..ownerID), "");
			rules = dec(rules);
			local nbOwner = 0;
			local usePrice = 0;
			for rule in string.gmatch(rules, "([^#]+)") do
				local ruleNickname, db_canUse, db_usePrice, db_canEdit = rule:match("([^,]+)|([^,]+)|([^,]+)|([^,]+)");
				if (db_canEdit == "1") then
					nbOwner = nbOwner + 1;
				end;
				if (ruleNickname == actionSender) then
					usePrice = tonumber(db_usePrice);
				end;
			end;

			usePrice = (usePrice/60)*useTime;
			if (usePrice > 0 and nbOwner > 0) then
				local mpmoney_xml = loadXMLFile("mpmoney_db", self.mpmoney_dbfilename);
				local sendPart = usePrice/nbOwner;
				if (sendPart > 0) then
					for rule in string.gmatch(rules, "([^#]+)") do
						local ruleNickname, db_canUse, db_usePrice, db_canEdit = rule:match("([^,]+)|([^,]+)|([^,]+)|([^,]+)");
						if (db_canEdit == "1") then
							if ruleNickname == actionSender then
								usePrice = usePrice - sendPart;
							else
								local targetBankMoney = self:getBankMoney(mpmoney_xml, ruleNickname);
								self:setBankMoney(mpmoney_xml, ruleNickname, targetBankMoney+sendPart);
								self:addInHistory(mpmoney_xml, "toBank", actionSender, ruleNickname, sendPart);
							end;
						end;
					end;
					if (usePrice > 0) then
						local sourceBankMoney = self:getBankMoney(mpmoney_xml, actionSender);
						self:setBankMoney(mpmoney_xml, actionSender, sourceBankMoney-usePrice);
					end;
				end;
				delete(mpmoney_xml);
			end;
		else
			self:fLog("## MPOwner ## ERROR : MPMoney database doesn't exist. Can not transfert money to money to other players. Install MPMoney, connect you to the game with a player and restart server.");
		end;
		
	elseif actionType == "canUse" then
	
		local rules = Utils.getNoNil(getXMLString(xml, "mpowner_db.ID"..actionData), "");
		local usePrice = 0;
		for rule in string.gmatch(rules, "([^#]+)") do
			local ruleNickname, db_canUse, db_usePrice, db_canEdit = rule:match("([^,]+)|([^,]+)|([^,]+)|([^,]+)");
			if (ruleNickname == actionSender) then
				usePrice = tonumber(db_usePrice);
				break;
			end;
		end;
		
		if (usePrice > 0) then
			local mpmoney_xml = loadXMLFile("mpmoney_db", self.mpmoney_dbfilename);
			local senderBankMoney = self:getBankMoney(mpmoney_xml, actionSender);
			if not (senderBankMoney > usePrice) then
				-- expulse
			end;
			delete(mpmoney_xml);
		end;
		
	elseif actionType == "saveRules" then
		
		local ownerID, rules = actionData:match("([^,]+),([^,]+)");
		rules = enc(rules);
		setXMLString(xml, "mpowner_db.ID"..ownerID, rules);
		saveXMLFile(xml);
		self:SendToServer(self.username, "needRefreshRules", ownerID);
		
    else
		self:fLog("## MPOwner ## ERROR : Invalid action '"..actionType.."' from "..actionSender);
    end;
	
	-- Close data file
    delete(xml);

end;


function MPOwner_ui:addInHistory(xml, transfertType, transfertSource, transfertTarget, transfertValue)
	
	self:addInHistory_2(xml, transfertSource, transfertType, transfertSource, transfertTarget, transfertValue);
	if not (transfertSource == transfertTarget) then
		self:addInHistory_2(xml, transfertTarget, transfertType, transfertSource, transfertTarget, transfertValue);
	end;
	
end;


function MPOwner_ui:addInHistory_2(xml, historyOwner, transfertType, transfertSource, transfertTarget, transfertValue)
	
	local edited = false;
	local curTimestamp = getDate("%y%m%d%H%M%S");
	local curTime = getDate("%d-%m-%y %H:%M");
	local infos = curTimestamp.."|"..curTime.."|"..transfertType.."|"..transfertSource.."|"..transfertTarget.."|"..transfertValue.."#";
	local tmpInfos = "";
	
	local clean_username = cleanup_string(tostring(historyOwner));
	local transferInfos = Utils.getNoNil(getXMLString(xml, "mpmoney_db."..tostring(clean_username).."_history"), "");
	transferInfos = dec(transferInfos);
	
	for playerInfos in string.gmatch(transferInfos, "([^#]+)") do
		local hy_timestamp, hy_datetime, hy_transfertType, hy_transfertSource, hy_transfertTarget, hy_transfertValue = playerInfos:match("([^,]+)|([^,]+)|([^,]+)|([^,]+)|([^,]+)|([^,]+)");
		if (string.len(tostring(hy_timestamp)) > 4) then
			if ((tonumber(curTimestamp) < (tonumber(hy_timestamp)+1800)) and transfertType == hy_transfertType and transfertSource == hy_transfertSource and transfertTarget == hy_transfertTarget) then
				tmpInfos = tmpInfos .. curTimestamp.."|"..curTime.."|"..transfertType.."|"..transfertSource.."|"..transfertTarget.."|"..(tonumber(transfertValue)+tonumber(hy_transfertValue)).."#";
				edited = true;
			else
				tmpInfos = tmpInfos .. hy_timestamp.."|"..hy_datetime.."|"..hy_transfertType.."|"..hy_transfertSource.."|"..hy_transfertTarget.."|"..hy_transfertValue.."#";
			end;
		end;
	end;

	if (edited) then
		transferInfos = enc(tmpInfos);
	else
		transferInfos = infos..transferInfos;
		transferInfos = enc(transferInfos);
	end;
	
	setXMLString(xml, "mpmoney_db."..tostring(clean_username).."_history", transferInfos);
	saveXMLFile(xml);
	
end;


function MPOwner_ui:getBankMoney(xml, username)
    --local clean_username = cleanup_string(tostring(username));
    local bankMoney = Utils.getNoNil(getXMLString(xml, "mpmoney_db."..tostring(username)), "MA==");
    bankMoney = dec(bankMoney);
    bankMoney = math.floor(tonumber(bankMoney));
    return bankMoney;

end;


function MPOwner_ui:setBankMoney(xml, username, money)
   -- local clean_username = cleanup_string(tostring(username));
    local enc_money = tostring(money);
    enc_money = enc(enc_money);
    setXMLString(xml, "mpmoney_db."..tostring(username), enc_money);
    saveXMLFile(xml);
    if (self:isMaster(username)) then
        self.bankMoney = money;
    end;
end;



function MPOwner_ui:keyEvent(unicode, sym, modifier, isDown)
end;


function MPOwner_ui:deleteMap()
end;



function MPOwner_ui:getKeyString(sym)

	if sym == Input.KEY_0 then
		return ("0");
	end;
	if sym == Input.KEY_1 then
		return ("1");
	end;
	if sym == Input.KEY_2 then
		return ("2");
	end;
	if sym == Input.KEY_3 then
		return ("3");
	end;
	if sym == Input.KEY_4 then
		return ("4");
	end;
	if sym == Input.KEY_5 then
		return ("5");
	end;
	if sym == Input.KEY_6 then
		return ("6");
	end;
	if sym == Input.KEY_7 then
		return ("7");
	end;
	if sym == Input.KEY_8 then
		return ("8");
	end;
	if sym == Input.KEY_9 then
		return ("9");
	end;
	
	if sym == Input.KEY_KP_0 then
		return ("0");
	end;
	if sym == Input.KEY_KP_1 then
		return ("1");
	end;
	if sym == Input.KEY_KP_2 then
		return ("2");
	end;
	if sym == Input.KEY_KP_3 then
		return ("3");
	end;
	if sym == Input.KEY_KP_4 then
		return ("4");
	end;
	if sym == Input.KEY_KP_5 then
		return ("5");
	end;
	if sym == Input.KEY_KP_6 then
		return ("6");
	end;
	if sym == Input.KEY_KP_7 then
		return ("7");
	end;
	if sym == Input.KEY_KP_8 then
		return ("8");
	end;
	if sym == Input.KEY_KP_9 then
		return ("9");
	end;
	
	return "";
	
end;


function MPOwner_ui:keyEvent(unicode, sym, modifier, isDown)

	if isDown then
		if self.HUDActive then
			if (self.params.currentEditPrice) then
				if sym == Input.KEY_backspace then
					self.params.usePrice = string.sub (self.params.usePrice, 0, string.len(self.params.usePrice)-1);
					if (self.params.usePrice == "") then
						self.params.usePrice = 0;
						self:saveRules();
					end;
				else
					local inputChar = self:getKeyString(sym);
					if not (inputChar == "") then
						self.params.usePrice = self.params.usePrice .. inputChar;
						self:saveRules();
					end;
				end;
			end;
		end;
		if not (self.userChecked) then
			--self:SendToServer("%All%", "checkMe","");
			self:SendToServer(self.username, "checkMe","");
			self.userChecked = true;
		end;
	end;


end;


addModEventListener(MPOwner_ui);

